@@ -1,6 +1,6 @@
 // -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; indent-tabs-mode: nil; -*-
-  
-  // we only include RcppArmadillo.h which pulls Rcpp.h in for us
+
+// we only include RcppArmadillo.h which pulls Rcpp.h in for us
 #include <RcppParallel.h>
 #include <RcppArmadillo.h>
 #include <Rcpp.h>
@@ -12,8 +12,7 @@ using namespace std;
 using namespace arma;
 
 arma::mat cubicmatrix(const vec z1){
-  
-  //vec h = diff(z1);
+
   vec h(z1.n_elem-1);
   for(unsigned i = 0; i < z1.n_elem-1; i++)
     h[i] = z1[i+1]-z1[i];
@@ -158,8 +157,8 @@ void spatmca_tau1(mat& G, mat& C, mat& Lambda2, const mat matrixinv, const int p
 }
 
 void spatmca_tau2(mat& G, mat& R, mat& C, mat& Lambda1, mat& Lambda2,
-           const mat matrixinv, double tau2u, double tau2v, const int p1,
-           const int p2, const double zeta, const int maxit, const double tol){
+                  const mat matrixinv, double tau2u, double tau2v, const int p1,
+                  const int p2, const double zeta, const int maxit, const double tol){
   
   int p = G.n_rows; 
   int K = G.n_cols;
@@ -225,8 +224,8 @@ struct spatmcacv_p: public RcppParallel::Worker {
   spatmcacv_p(const mat& X, const mat& Y, const int K, const mat& Omega1, const mat& Omega2,const vec& tau1u, 
               const vec &tau1v, const vec& nk, const int p1,const int p2,const int maxit, 
               const double tol, cube& output, cube& S12train, cube& S12traint, cube& S12valid, cube& S1221, cube& S2112, vec& zetatemp, cube& Ucv, cube& Vcv, cube& Lmbd12cv, cube& Lmbd22cv) : X(X), Y(Y), K(K), Omega1(Omega1), Omega2(Omega2), tau1u(tau1u), tau1v(tau1v),
-  nk(nk), p1(p1), p2(p2), maxit(maxit),tol(tol), output(output), S12train(S12train), S12traint(S12traint), S12valid(S12valid), S1221(S1221), S2112(S2112),zetatemp(zetatemp),
-  Ucv(Ucv), Vcv(Vcv), Lmbd12cv(Lmbd12cv), Lmbd22cv(Lmbd22cv){}
+              nk(nk), p1(p1), p2(p2), maxit(maxit),tol(tol), output(output), S12train(S12train), S12traint(S12traint), S12valid(S12valid), S1221(S1221), S2112(S2112),zetatemp(zetatemp),
+              Ucv(Ucv), Vcv(Vcv), Lmbd12cv(Lmbd12cv), Lmbd22cv(Lmbd22cv){}
   void operator()(std::size_t begin, std::size_t end) {
     arma::mat Ip;
     Ip.eye(Y.n_cols,Y.n_cols);
@@ -266,7 +265,7 @@ struct spatmcacv_p: public RcppParallel::Worker {
         for(uword  j = 0; j < tau1v.n_elem; j++){    
           Theta.submat(p1,p1,p1+p2-1,p1+p2-1) = -tau1v[j]*Omega2;
           if(j == 0 && i ==0){
-             output(i,j,k) = norm((S12valid.slice(k)-G.rows(0,p1-1)*diagmat(max(zero,svdtemp.subvec(0,K-1)))*G.rows(p1,p1+p2-1).t()),"fro");
+            output(i,j,k) = norm((S12valid.slice(k)-G.rows(0,p1-1)*diagmat(max(zero,svdtemp.subvec(0,K-1)))*G.rows(p1,p1+p2-1).t()),"fro");
           }
           else{//if(j !=0 && i != 0){ 
             matrixinv =  0.5*arma::inv_sympd(zetatemp[k]*Ip-Theta);
@@ -314,10 +313,10 @@ struct spatmcacv_pp: public RcppParallel::Worker {
   cube& Lmbd12cv;
   cube& Lmbd22cv;
   spatmcacv_pp(const mat& X, const mat& Y, const int K, const mat& Omega1, const mat& Omega2,const vec& tau1u, 
-              const vec &tau1v, const vec& nk, const int p1,const int p2,const int maxit, 
-              const double tol, cube& output, cube& S12train, cube& S12traint, cube& S12valid, cube& S1221, cube& S2112, vec& zetatemp, cube& Ucv, cube& Vcv,cube& Lmbd12cv, cube& Lmbd22cv) : X(X), Y(Y), K(K), Omega1(Omega1), Omega2(Omega2), tau1u(tau1u), tau1v(tau1v),
-  nk(nk), p1(p1), p2(p2), maxit(maxit),tol(tol), output(output), S12train(S12train), S12traint(S12traint), S12valid(S12valid), S1221(S1221), S2112(S2112),zetatemp(zetatemp),
-  Ucv(Ucv), Vcv(Vcv), Lmbd12cv(Lmbd12cv), Lmbd22cv(Lmbd22cv){}
+               const vec &tau1v, const vec& nk, const int p1,const int p2,const int maxit, 
+               const double tol, cube& output, cube& S12train, cube& S12traint, cube& S12valid, cube& S1221, cube& S2112, vec& zetatemp, cube& Ucv, cube& Vcv,cube& Lmbd12cv, cube& Lmbd22cv) : X(X), Y(Y), K(K), Omega1(Omega1), Omega2(Omega2), tau1u(tau1u), tau1v(tau1v),
+               nk(nk), p1(p1), p2(p2), maxit(maxit),tol(tol), output(output), S12train(S12train), S12traint(S12traint), S12valid(S12valid), S1221(S1221), S2112(S2112),zetatemp(zetatemp),
+               Ucv(Ucv), Vcv(Vcv), Lmbd12cv(Lmbd12cv), Lmbd22cv(Lmbd22cv){}
   void operator()(std::size_t begin, std::size_t end) {
     arma::mat Ip;
     Ip.eye(Y.n_cols,Y.n_cols);
@@ -383,7 +382,7 @@ struct spatmcacv_p2: public RcppParallel::Worker {
                int K, double tau1u, double tau1v, const vec& tau2u, const vec& tau2v,
                const vec& nk, int p1, int p2, int maxit, double tol, cube& output):
     S12train(S12train),S12traint(S12traint), S12valid(S12valid), S1221(S1221), S2112(S2112), zetatemp(zetatemp), Ucv(Ucv), Vcv(Vcv), Lmbd12cv(Lmbd12cv),
-  Lmbd22cv(Lmbd22cv), Omega1(Omega1), Omega2(Omega2), K(K), tau1u(tau1u), tau1v(tau1v), tau2u(tau2u), tau2v(tau2v), nk(nk), p1(p1), p2(p2), maxit(maxit), tol(tol), output(output){}
+    Lmbd22cv(Lmbd22cv), Omega1(Omega1), Omega2(Omega2), K(K), tau1u(tau1u), tau1v(tau1v), tau2u(tau2u), tau2v(tau2v), nk(nk), p1(p1), p2(p2), maxit(maxit), tol(tol), output(output){}
   void operator()(std::size_t begin, std::size_t end) { 
     for(std::size_t k = begin; k < end; k++){
       arma::mat G(p1+p2,K), C(p1+p2,K), R(p1+p2,K), Gamma1(p1+p2,K), Gamma2(p1+p2,K), Gold(p1+p2,K), Cold(p1+p2,K), Rold(p1+p2,K), Gamma1old(p1+p2,K), Gamma2old(p1+p2,K), Ip;
@@ -424,8 +423,7 @@ struct spatmcacv_p2: public RcppParallel::Worker {
         }
       }
       if(tau1u == 0&& tau1v == 0){
-       // printf("ok");
-          matrixinv =  0.5*arma::inv_sympd(zetatemp[k]*Ip-Theta);
+        matrixinv =  0.5*arma::inv_sympd(zetatemp[k]*Ip-Theta);
       }
       Rold = Gold;
       Gamma1old = 0*Gamma2old;
@@ -478,30 +476,28 @@ List spatmcacv_rcpp(NumericMatrix  sxr, NumericMatrix  syr, NumericMatrix Xr, Nu
   arma::mat Omega1, Omega2, out, out2;
   out.zeros(tau1u.n_elem, tau1v.n_elem);
   
- // printf("welcome");
- 
- if(max(tau1u)==0){
-   Omega1.eye(p,p);
- }
- else{
-   if(d == 2){
-     Omega1 = tpmatrix(sx);
-   }
-   else{
-     Omega1 = cubicmatrix(sx);
-   }
- }
- if(max(tau1v)==0){
-   Omega2.eye(q,q);
- }
- else{
-   if(d == 2){
-     Omega2 = tpmatrix(sy);
-   }
-   else{
-     Omega2 = cubicmatrix(sy);
-   }
- }
+  if(max(tau1u)==0){
+    Omega1.eye(p,p);
+  }
+  else{
+    if(d == 2){
+      Omega1 = tpmatrix(sx);
+    }
+    else{
+      Omega1 = cubicmatrix(sx);
+    }
+  }
+  if(max(tau1v)==0){
+    Omega2.eye(q,q);
+  }
+  else{
+    if(d == 2){
+      Omega2 = tpmatrix(sy);
+    }
+    else{
+      Omega2 = cubicmatrix(sy);
+    }
+  }
   arma::mat S12est = X.t()*Y/n;
   arma::mat S12estt = S12est.t();
   arma::mat Utempest, Vtempest;
@@ -529,7 +525,7 @@ List spatmcacv_rcpp(NumericMatrix  sxr, NumericMatrix  syr, NumericMatrix Xr, Nu
     cvtau1v= max(tau1v);
     Thetaest.submat(0,0,p-1,p-1) = -cvtau1u*Omega1; 
     Thetaest.submat(p,p,p+q-1,p+q-1) = -cvtau1v*Omega2;
-   
+    
     if(cvtau1u !=0){
       arma::mat M1 = arma::inv_sympd(2*zeta*Ip.submat(0,0,p-1,p-1)-2*Thetaest.submat(0,0,p-1,p-1) - S1221est/(2*zeta));
       matrixinv.submat(0,0,p-1,p-1) = M1;
@@ -540,7 +536,7 @@ List spatmcacv_rcpp(NumericMatrix  sxr, NumericMatrix  syr, NumericMatrix Xr, Nu
     }
     if(cvtau1u == 0 && cvtau1v ==0){
       matrixinv=0.5*arma::inv_sympd(zeta*Ip-Thetaest);
-      }
+    }
     else if(cvtau1v !=0){
       if(cvtau1u == 0){
         arma::mat M2 = arma::inv_sympd(2*zeta*Ip.submat(p,p,p+q-1,p+q-1)-2*Thetaest.submat(p,p,p+q-1,p+q-1) - S2112est/(2*zeta));
@@ -557,17 +553,15 @@ List spatmcacv_rcpp(NumericMatrix  sxr, NumericMatrix  syr, NumericMatrix Xr, Nu
     }
     if(tau2u.n_elem != 1 || tau2v.n_elem != 1){
       spatmcacv_pp spatmcacv_pp(X, Y, K, Omega1, Omega2, tau1u, tau1v, nk, p, q,maxit, 
-                              tol, cv, S12train, S12traint, S12valid, S1221, S2112,zetatemp, Ucv, Vcv, Lmbd12cv, Lmbd22cv);
+                                tol, cv, S12train, S12traint, S12valid, S1221, S2112,zetatemp, Ucv, Vcv, Lmbd12cv, Lmbd22cv);
       RcppParallel::parallelFor(0, M, spatmcacv_pp);
     }
     out.zeros(1);
   }
   else{
-  //  printf("cc");
     spatmcacv_p spatmcacv_p(X, Y, K, Omega1, Omega2, tau1u, tau1v, nk, p, q,maxit, 
                             tol, cv, S12train, S12traint, S12valid, S1221, S2112,zetatemp, Ucv, Vcv,Lmbd12cv, Lmbd22cv);
     RcppParallel::parallelFor(0, M, spatmcacv_p);
-  //  printf("ddd");
     uword row1, col1;
     for(uword m = 0; m < M; m++)
       out += cv.slice(m)/M;
@@ -585,12 +579,12 @@ List spatmcacv_rcpp(NumericMatrix  sxr, NumericMatrix  syr, NumericMatrix Xr, Nu
       spatmca_tau1(Gest, Cest, Gamma2est, matrixinv, p, q, zeta, maxit, tol);  
     }
     if(cvtau1u == 0 && cvtau1v ==0){
-   //   arma::mat A = diagmat(1/(pow(zeta,2)-pow(SPhiest.subvec(0,K-1),2)));
-  //    matrixinv.submat(0,0,p-1,p-1) = 2*zeta*Gest.rows(0,p-1)*A*Gest.rows(0,p-1).t();
-  //    matrixinv.submat(0,p,p-1,p+q-1) = Gest.rows(0,p-1)*diagmat(SPhiest.subvec(0,K-1))*A*Gest.rows(p,p+q-1).t();
-  //    matrixinv.submat(p,0,p+q-1,p-1) = matrixinv.submat(0,p,p-1,p+q-1).t();
-  //    matrixinv.submat(p,p,p+q-1,p+q-1) = 2*zeta*Gest.rows(p,p+q-1)*A*Gest.rows(p,p+q-1).t();;
-       matrixinv=0.5*arma::inv_sympd(zeta*Ip-Thetaest);
+      //   arma::mat A = diagmat(1/(pow(zeta,2)-pow(SPhiest.subvec(0,K-1),2)));
+      //    matrixinv.submat(0,0,p-1,p-1) = 2*zeta*Gest.rows(0,p-1)*A*Gest.rows(0,p-1).t();
+      //    matrixinv.submat(0,p,p-1,p+q-1) = Gest.rows(0,p-1)*diagmat(SPhiest.subvec(0,K-1))*A*Gest.rows(p,p+q-1).t();
+      //    matrixinv.submat(p,0,p+q-1,p-1) = matrixinv.submat(0,p,p-1,p+q-1).t();
+      //    matrixinv.submat(p,p,p+q-1,p+q-1) = 2*zeta*Gest.rows(p,p+q-1)*A*Gest.rows(p,p+q-1).t();;
+      matrixinv=0.5*arma::inv_sympd(zeta*Ip-Thetaest);
     }
     else if(cvtau1v !=0){
       if(cvtau1u == 0){
@@ -646,4 +640,218 @@ List spatmcacv_rcpp(NumericMatrix  sxr, NumericMatrix  syr, NumericMatrix Xr, Nu
   return List::create(Named("cv1") = out, Named("cv2") = out2, Named("Uest") = Gest.rows(0,p-1), Named("Vest") = Gest.rows(p,p+q-1), Named("Dest")= D, Named("cvtau1u") = cvtau1u,Named("cvtau2u") = cvtau2u, Named("cvtau1v") = cvtau1v, Named("cvtau2v") = cvtau2v);
 }
 
+using namespace arma; 
+using namespace Rcpp;
+struct spatmcacv_pall: public RcppParallel::Worker {
+  const mat& X;
+  const mat& Y;
+  const int K;
+  const mat& Omega1;
+  const mat& Omega2;
+  const vec& tau1u;
+  const vec& tau1v;
+  const vec& tau2u;
+  const vec& tau2v;
+  const vec& nk;
+  const int p1;
+  const int p2;
+  const int maxit;
+  const double tol;
+  cube& output;
+  spatmcacv_pall(const mat& X, const mat& Y, const int K, const mat& Omega1, const mat& Omega2,const vec& tau1u, 
+                 const vec &tau1v,const vec& tau2u, const vec &tau2v, const vec& nk, const int p1,const int p2,const int maxit, 
+                 const double tol, cube& output) : X(X), Y(Y), K(K), Omega1(Omega1), Omega2(Omega2), tau1u(tau1u), tau1v(tau1v),
+                 tau2u(tau2u), tau2v(tau2v), nk(nk), p1(p1), p2(p2), maxit(maxit),tol(tol), output(output){}
+  void operator()(std::size_t begin, std::size_t end) {
+    arma::mat Ip;
+    Ip.eye(Y.n_cols,Y.n_cols);
+    for(std::size_t k = begin; k < end; k++){
+      arma::mat Uoldtemp, Voldtemp, Goldtemp, G(p1+p2,K), R(p1+p2,K),C(p1+p2,K), Gamma1(p1+p2,K),
+      Gamma2(p1+p2,K), Gold(p1+p2,K), Rold(p1+p2,K), Cold(p1+p2,K), Gamma1old(p1+p2,K), Gamma2old(p1+p2,K),
+      Gold0(p1+p2,K), Rold0(p1+p2,K), Cold0(p1+p2,K), Gamma1old0(p1+p2,K), Gamma2old0(p1+p2,K),
+      Gold2(p1+p2,K), Rold2(p1+p2,K), Cold2(p1+p2,K), Gamma1old2(p1+p2,K), Gamma2old2(p1+p2,K),
+      Gold3(p1+p2,K), Rold3(p1+p2,K), Cold3(p1+p2,K), Gamma1old3(p1+p2,K), Gamma2old3(p1+p2,K),
+      S12train(p1, p2), S12traint(p2, p1), S12valid(p1, p2), matrixinv(p1+p2, p1+p2), Ip;
+      
+      vec svdtemp, D;
+      arma::mat Ytrain = Y.rows(arma::find(nk!=(k+1)));
+      arma::mat Yvalid = Y.rows(arma::find(nk==(k+1)));
+      arma::mat Xtrain = X.rows(arma::find(nk!=(k+1)));
+      arma::mat Xvalid = X.rows(arma::find(nk==(k+1)));
+      arma::mat Theta(p1+p2,p1+p2);
+      S12train = (Xtrain.t())*Ytrain/Xtrain.n_rows;
+      S12valid = Xvalid.t()*Yvalid/Xvalid.n_rows;
+      svd_econ(Uoldtemp, svdtemp,Voldtemp, S12train);
+      double zetatemp = 10*max(svdtemp);
+      Gold.rows(0,p1-1) = Uoldtemp.cols(0,K-1);
+      Rold.rows(0,p1-1) = Uoldtemp.cols(0,K-1);
+      Cold.rows(0,p1-1) = Uoldtemp.cols(0,K-1);
+      Gold.rows(p1,p1+p2-1) = Voldtemp.cols(0,K-1);
+      Rold.rows(p1,p1+p2-1) = Voldtemp.cols(0,K-1);
+      Cold.rows(p1,p1+p2-1) = Voldtemp.cols(0,K-1);
+      Gamma2old.rows(0,p1-1)= S12train*Gold.rows(p1,p1+p2-1);
+      S12traint = S12train.t();
+      Gamma2old.rows(p1,p1+p2-1) = S12traint*Gold.rows(0,p1-1);
+      
+      Gold0 = Gold2 = Gold3 = Gold;
+      Rold0 = Rold2 = Rold3 = Rold;
+      Cold0 = Cold2 = Cold3 = Cold;
+      Gamma2old0 = Gamma2old2 = Gamma2old3 = Gamma2old;
+      
+      Ip.eye(p1+p2,p1+p2);
+      Gamma1old.zeros(p1+p2, K);
+      Gamma1old0 = Gamma1old2 = Gamma1old3 = Gamma1old;
+      Theta.submat(0, p1, p1-1, p1+p2-1) = S12train/2;
+      Theta.submat(p1, 0, p1+p2-1, p1-1) = S12traint/2;
+      vec zero;
+      zero.zeros(K);
+      for(uword  i = 0; i < tau1u.n_elem; i++){
+        G = Gold0;
+        R = Rold0;
+        C = Cold0;
+        Gamma1 = Gamma1old0;
+        Gamma2 = Gamma2old0;
+        Theta.submat(0, 0, p1-1, p1-1) = -tau1u[i]*Omega1;
+        for(uword  j = 0; j < tau1v.n_elem; j++){    
+          Theta.submat(p1, p1, p1+p2-1, p1+p2-1) = -tau1v[j]*Omega2;
+          
+            matrixinv = 0.5*arma::inv_sympd(zetatemp*Ip-Theta);
+            vec zero;
+            zero.zeros(K);
+            for(uword  l = 0; l < tau2u.n_elem; l++){
+              G = Gold3;
+              R = Rold3;
+              C = Cold3;
+              Gamma1 = Gamma1old3;
+              Gamma2 = Gamma2old3;
+              for(uword  m = 0; m < tau2v.n_elem; m++){
+                spatmca_tau2(G, R, C, Gamma1, Gamma2, matrixinv, tau2u[l], tau2v[m], p1, p2, zetatemp, maxit, tol);
+                D = max(zero, diagvec(G.rows(0, p1-1).t()*S12train*G.rows(p1, p1+p2-1)));
+                output(tau2u.n_elem*i + l, tau2v.n_elem*j + m, k) = 
+                  norm((S12valid-G.rows(0, p1-1)*diagmat(D)*G.rows(p1, p1+p2-1).t()),"fro");
+                if(l == 0 && m == 0){
+                  Gold2 = G;
+                  Rold2 = R;
+                  Cold2 = C;
+                  Gamma1old2 = Gamma1;
+                  Gamma2old2 = Gamma2;
+                }
+                if(m == 0){
+                  Gold3 = G;
+                  Rold3 = R;
+                  Cold3 = C;
+                  Gamma1old3 = Gamma1;
+                  Gamma2old3 = Gamma2;
+                }
+              }
+            }
+          
+          if(j == 0){ 
+            Gold0 = Gold2;
+            Rold0 = Rold2;
+            Cold0 = Cold2;
+            Gamma1old0 = Gamma1old2;
+            Gamma2old0 = Gamma2old2;
+          }
+          Gold = Gold2;
+          Rold = Rold2;
+          Cold = Cold2;
+          Gamma1old = Gamma1old2;
+          Gamma2old = Gamma2old2;
+        }
+      }
+    }
+  }
+};
+
+
+
+
+
+using namespace Rcpp;
+using namespace arma;
+using namespace std;
+// [[Rcpp::export]]
+List spatmcacvall_rcpp(NumericMatrix  sxr, NumericMatrix  syr, NumericMatrix Xr, NumericMatrix Yr, int M, int K, 
+                       NumericVector  tau1ur, NumericVector  tau2ur, NumericVector  tau1vr, NumericVector  tau2vr,  NumericVector  nkr, int maxit, double tol, NumericVector  l2ur,NumericVector  l2vr) {
+  int n = Yr.nrow(), p = Xr.ncol(), q = Yr.ncol(), d = sxr.ncol();
+  arma::mat X(Xr.begin(), n, p, false);
+  arma::mat Y(Yr.begin(), n, q, false);
+  arma::mat sx(sxr.begin(), p, d, false);
+  arma::mat sy(syr.begin(), q, d, false);
+  colvec tau1u(tau1ur.begin(), tau1ur.size(), false);
+  colvec tau2u(tau2ur.begin(), tau2ur.size(), false);
+  colvec tau1v(tau1vr.begin(), tau1vr.size(), false);
+  colvec tau2v(tau2vr.begin(), tau2vr.size(), false);
+  colvec nk(nkr.begin(), nkr.size(), false);
+  colvec l2u(l2ur.begin(), l2ur.size(), false);
+  colvec l2v(l2vr.begin(), l2vr.size(), false);
+  arma::cube cv(tau1u.n_elem*tau2u.n_elem, tau1v.n_elem*tau2v.n_elem,M);
+  arma::mat Omega1, Omega2, out, out2;
+  out.zeros(tau1u.n_elem, tau1v.n_elem);
+  
+  
+  if(d == 2){
+    Omega1 = tpmatrix(sx);
+    Omega2 = tpmatrix(sy);
+  }
+  else{
+    Omega1 = cubicmatrix(sx);
+    Omega2 = cubicmatrix(sy);
+  }
+  
+  arma::mat S12est = X.t()*Y/n;
+  arma::mat S12estt = S12est.t();
+  arma::mat Utempest, Vtempest;
+  vec SPhiest;
+  arma::svd_econ(Utempest, SPhiest, Vtempest, S12est);
+  double zeta = 10*max(SPhiest);
+  arma::mat Gest(p+q,K), Cest(p+q,K), Gamma2est(p+q,K), Thetaest(p+q,p+q), matrixinv(p+q,p+q);
+  Gest.rows(0,p-1) = Utempest.cols(0,K-1);
+  Cest.rows(0,p-1) = Utempest.cols(0,K-1);
+  Gest.rows(p,p+q-1) = Vtempest.cols(0,K-1);
+  Cest.rows(p,p+q-1)  = Vtempest.cols(0,K-1);
+  Gamma2est.rows(0,p-1) = S12est*Vtempest.cols(0,K-1);
+  Gamma2est.rows(p,p+q-1) = S12estt*Utempest.cols(0,K-1);
+  Thetaest.submat(0,p,p-1,p+q-1) = S12est/2;
+  Thetaest.submat(p,0,p+q-1,p-1) = S12estt/2;
+  mat S1221est = S12est*S12estt, S2112est = S12estt*S12est;
+  double cvtau1u, cvtau1v, cvtau2u, cvtau2v;
+  arma::mat Ip;
+  Ip.eye(p+q,p+q);
+ 
+  out.zeros(tau1u.n_elem*tau2u.n_elem, tau1v.n_elem*tau2v.n_elem);  
+  spatmcacv_pall spatmcacv_pall(X, Y, K, Omega1, Omega2, tau1u, 
+                                tau1v, tau2u, tau2v, nk, p, q, maxit, 
+                                tol, cv);
+  RcppParallel::parallelFor(0, M, spatmcacv_pall);
+
+  uword row, col;
+  for(uword m = 0; m < M; m++)
+    out += cv.slice(m)/M;
+  out.min(row, col);
+
+  cvtau1u = tau1u[floor(row/tau2u.n_elem)];
+  cvtau2u = tau2u[row % tau2u.n_elem];
+  cvtau1v = tau1v[floor(col/tau2v.n_elem)];
+  cvtau2v = tau2v[col % tau2v.n_elem];
+
+  Thetaest.submat(0,0,p-1,p-1) = -cvtau1u*Omega1; 
+  Thetaest.submat(p,p,p+q-1,p+q-1) = -cvtau1v*Omega2;
+  
+  matrixinv =  arma::inv_sympd(2*zeta*Ip-2*Thetaest);
+  spatmca_tau1(Gest, Cest, Gamma2est, matrixinv, p, q, zeta, maxit, tol);
+  
+  arma::mat Rest = Gest;
+  arma::mat Gamma1est = 0*Gamma2est;
+  
+  for(uword  i = 0; i <= row % tau2u.n_elem; i++)
+    spatmca_tau2(Gest, Rest, Cest, Gamma1est, Gamma2est, matrixinv, tau2u[i], 0, p, q, zeta, maxit,tol);
+  for(uword  i = 0; i <= col % tau2v.n_elem; i++)
+    spatmca_tau2(Gest, Rest, Cest, Gamma1est, Gamma2est, matrixinv, cvtau2u, tau2v[i], p, q, zeta, maxit,tol);    
+  arma::vec zeros;
+  zeros.zeros(K);
+  arma::vec D = max(zeros, diagvec(Gest.rows(0,p-1).t()*S12est*Gest.rows(p,p+q-1)));
+  return List::create(Named("cvall") = out, Named("Uest") = Gest.rows(0,p-1), Named("Vest") = Gest.rows(p,p+q-1), Named("Dest")= D, Named("cvtau1u") = cvtau1u,Named("cvtau2u") = cvtau2u, Named("cvtau1v") = cvtau1v, Named("cvtau2v") = cvtau2v);
+}
 
